# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimList,
    HprimMultiple,
    HprimString,
    Segment,
)


class LiaisonSegmentH(HeaderSegment):

    sender_id = HprimString(5)
    receiver_id = HprimString(10)
    version = HprimString(13, default="1")
    patients = HprimForeign("P", "header")
    date = HprimDate(14)
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class LiaisonSegmentP(Segment):

    patient_id = HprimString(4, max_length=20)
    name = HprimString((6, 1), max_length=20)
    first_name = HprimString((6, 2), max_length=20)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    sex = HprimString(9)
    physician_id = HprimString(14, max_length=6)
    orders = HprimForeign("O", "patient")


class LiaisonSegmentQ(Segment):
    range_id = HprimList(3)
    test_id = HprimString(5, default="ALL")
    status = HprimString(13, default="O")


class LiaisonSegmentO(Segment):

    specimen_id = HprimString(3, max_length=20)
    tests = HprimList(
        5,
        basefield=HprimMultiple(0, code=HprimString(4), dilution_ratio=HprimString(5)),
    )

    # S: Stat
    # N: Normal
    priority = HprimString(6)

    request_date = HprimDate(7)

    # O:
    # F:
    # I:
    # C:
    # X:
    report_type = HprimString(26)

    results = HprimForeign("R", "order")


class LiaisonSegmentR(Segment):

    test = HprimString((3, 4))
    value = HprimString(4)
    units = HprimString(5)

    # L: Below normal range
    # H: Above normal range
    # <: Below assay range
    # >: Above assay range
    # N: Within normal range
    abnormal_flag = HprimList(7)

    # F: Final
    # I:
    # C:
    # X: Rejection
    status = HprimString(9)
    date = HprimDate(13)
    instrument_id = HprimString(14)


class LiaisonSegmentC(Segment):

    # I: Clinical instrument system
    source = HprimString(3)
    text = HprimList(4)
    # I: Instrument flag comments
    comment_type = HprimString(5)


class LiaisonSegmentL(Segment):

    # N: Normal termination
    # I: No info available
    code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {"code": "liaison", "label": "Liaison"}
    SEGMENT_CLASS_MAP = {
        "H": LiaisonSegmentH,
        "P": LiaisonSegmentP,
        "O": LiaisonSegmentO,
        "Q": LiaisonSegmentQ,
        "C": LiaisonSegmentC,
        "R": LiaisonSegmentR,
        "L": LiaisonSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.Option("liaison_sender", label="Code émetteur", default="MCA"),
        AstmMessageV2.Option(
            "liaison_receiver", label="Code récepteur", default="Liaison"
        ),
        AstmMessageV2.BooleanOption(
            "liaison_ignore_flag_qc",
            label="Ignorer les alarmes pour les CQ",
            default=False,
        ),
    ]

    def __init__(self, **kwargs):
        self.liaison_sender = kwargs.pop("liaison_sender", "MCA")
        self.liaison_receiver = kwargs.pop("liaison_receiver", "Liaison")
        self.liaison_ignore_flag_qc = kwargs.pop("liaison_ignore_flag_qc", False)
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = LiaisonSegmentH(type_segment="H", **kwargs)
        message.end = LiaisonSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        msg = self.create(
            sender_id=self.liaison_sender,
            receiver_id=self.liaison_receiver,
            date=context["now"],
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: "N"},
            priority={"S": "S", worklist.ELSE: "N"},
        ):
            seg_p = msg.patients.add(
                patient_id=patient.patient_id,
                name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
            )
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    specimen_id=sample.sample_id,
                    priority=priority,
                    action_code=action,
                    request_date=sample.request_date,
                    report_type="O",
                )
                for order in sample.orders:
                    t = seg_o.tests.add()
                    t.code = order.code
                    t.dilution_ratio = order.dilution

        if msg.patients:
            return msg
        elif context["query"]:
            msg.end.code = "I"
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                for q_id in seg_q.range_id:
                    worklist.get_or_create(sample_id=q_id, type="patient")
            return worklist

        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                is_qc = not seg_o.specimen_id.isdigit()
                if is_qc:
                    sample = worklist.get_or_create(
                        type="control",
                        sample_id=seg_o.specimen_id,
                        control_lot=seg_o.specimen_id,
                    )
                else:
                    sample = worklist.get_or_create(
                        type="patient", sample_id=seg_o.specimen_id
                    )
                for seg_r in seg_o.results:
                    if is_qc and seg_r.status != "F":
                        continue
                    if is_qc and self.liaison_ignore_flag_qc:
                        alarmes = []
                    else:
                        alarmes = []
                        if seg_r.abnormal_flag:
                            alarmes = seg_r.abnormal_flag
                        for seg_c in seg_r.commentaires:
                            alarmes.extend(seg_c.text)
                    sample.results.add(
                        code=seg_r.test,
                        date=seg_r.date,
                        flags=alarmes,
                        value=seg_r.value,
                    )
        return worklist
