# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimString,
    Segment,
)

logger = logging.getLogger("driverng.multistat")


class MultistatSegmentH(HeaderSegment):
    sender_name = HprimString(5)
    processing_id = HprimString(12, default="P")
    version = HprimString(13, default="LIS2A2")
    date = HprimDate(14)

    orders = HprimForeign("O", "header")
    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class MultistatSegmentP(Segment):
    patient_id = HprimString(3)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)

    orders = HprimForeign("O", "patient")


class MultistatSegmentO(Segment):
    sample_id = HprimString(3)
    test_code = HprimString((5, 4))
    priority = HprimString(6)
    request_date = HprimDate(7)
    action_code = HprimString(12)
    specimen_id = HprimString((16, 1))
    specimen_name = HprimString((16, 2))
    report_type = HprimString(26)

    results = HprimForeign("R", "order")


class MultistatSegmentR(Segment):
    test_code = HprimString((3, 4))
    value = HprimString(4)
    reference_range = HprimString(6)
    flags = HprimString(7)
    result_status = HprimString(9)
    operator = HprimString(11)
    start_date = HprimDate(12)
    end_date = HprimDate(13)
    instrument_id = HprimString(14)


class MultistatSegmentQ(Segment):
    sample_id = HprimString(3)
    status_code = HprimString(13)


class MultistatSegmentL(Segment):
    terminaison_code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {"code": "multistat", "label": "RANDOX Multistat"}

    SEGMENT_CLASS_MAP = {
        "H": MultistatSegmentH,
        "P": MultistatSegmentP,
        "O": MultistatSegmentO,
        "Q": MultistatSegmentQ,
        "R": MultistatSegmentR,
        "L": MultistatSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.Option(
            "multistat_sender_name",
            label="Paramétrer le nom de l'émetteur du segment H",
            default="MCA",
        )
    ]

    def __init__(self, **kwargs):
        self.multistat_sender_name = kwargs.pop("multistat_sender_name", "MCA")
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = MultistatSegmentH(type_segment="H", **kwargs)
        message.end = MultistatSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        message = self.create(
            sender_name=self.multistat_sender_name, date=context["now"]
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"A": "S", "S": "S", worklist.ELSE: "S"},
            action={"del": worklist.IGNORE, "add": "N", "new": "N"},
        ):
            seg_p = message.patients.add(
                patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
            )
            for sample, action, priority in patient.samples:
                for order in sample.orders:
                    seg_p.orders.add(
                        sample_id=sample.sample_id,
                        test_code=order.code,
                        priority=priority,
                        request_date=sample.request_date,
                    )
        if message.patients:
            return message
        elif context["query"]:
            seg_p = message.patients.add()
            for seg_q in context["query"].queries:
                seg_p.orders.add(sample_id=seg_q.sample_id, report_type="Y")
            return message

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(type="patient", sample_id=seg_q.sample_id)
        else:
            worklist = Worklist(context="result")
            for seg_o in message.orders:
                if seg_o.action_code == "Q":
                    sample = worklist.get_or_create(
                        sample_id=seg_o.sample_id,
                        control_lot=seg_o.sample_id,
                        type="control",
                    )
                else:
                    sample = worklist.get_or_create(
                        sample_id=seg_o.sample_id,
                        specimen=seg_o.specimen_id,
                        type="patient",
                    )
                for seg_r in seg_o.results:
                    flags = []
                    if seg_r.flags:
                        flags.append(seg_r.flags)
                    sample.results.add(
                        code=seg_r.test_code,
                        value=seg_r.value,
                        date=seg_r.end_date,
                        instrument=seg_r.instrument_id,
                        operator=seg_r.operator,
                        flags=flags,
                    )

        return worklist
